==> lvm-0.3alpha.tar.gz: LVM/0.3alpha/tools/lib/pv_reserve_release_pe.c <==
/*
 * Author: Heinz Mauelshagen, Germany
 *
 * March 1997
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <liblvm.h>

int pv_reserve_pe ( pv_t *pv, disk_pe_t *lv_pe, uint *pe,
                    pe_t *pe_p, uint allocation, int new) {
   int p = 0;
   int np = 0;

#ifdef DEBUG
   debug ( "pv_reserve_pe -- CALLED: pv->pv_dev: %02d:%02d  lv_num: %d"
            "  le_num: %d  pv->pe_total: %lu\n",
             MAJOR(pv->pv_dev),
             MINOR(pv->pv_dev),
             lv_pe->lv_num,
             lv_pe->le_num,
             pv->pe_total);
#endif

   if ( pv == NULL || lv_pe == NULL || lv_pe->lv_num > MAX_LV ||
        pe == NULL || *pe == 0 || pe_p == NULL ||
        ( allocation != 0 && allocation != LV_CONTIGUOUS) ||
        ( new != TRUE && new != FALSE)) return -LVM_EPARA;

   p = np = 0;
   if ( ( allocation & LV_CONTIGUOUS) && new != TRUE) {
      p = pv->pe_total - 1;
      while ( pv->pe[p].lv_num != lv_pe->lv_num && p > -1) p--;
      if ( p > 0) p++;
      else return -LVM_EPV_RESERVE_PE_LV; /* this LV not on this PV */
   }

   for ( p = 0; p < pv->pe_total && np < *pe; p++) {
      if ( pv->pe[p].lv_num == 0) {
#ifdef DEBUG
         debug ( "pv_reserve_pe -- empty PE %d\n", p);
#endif
         if ( ( allocation & LV_CONTIGUOUS) && new != TRUE &&
                pv->pe[p-1].lv_num != lv_pe->lv_num) return -LVM_ESIZE;
         pv->pe[p].lv_num = lv_pe->lv_num;
         pv->pe[p].le_num = lv_pe->le_num;
         lv_pe->le_num++;
         pv->pe_allocated++;
         pe_p->dev = pv->pv_dev;
         pe_p->pe = ( p * pv->pe_size) + LVM_DISK_SIZE ( pv) / SECTOR_SIZE;
         pe_p++;
         np++;
      }
   }
   *pe -= np;

#ifdef DEBUG
   debug ( "pv_reserve_pe -- LEAVING\n");
#endif
   return 0;
}


int pv_release_pe ( vg_t *vg, disk_pe_t *lv_pe, uint *pe, uint stripes) {
   int l = 0;
   int pv_num = 0;
   uint dest = 0;
   uint i = 0;
   uint j = 0;
   uint length = 0;
   uint np = 0;
   uint offset = 0;
   uint pe_index = 0;
   uint source = 0;
   uint p = 0;

#ifdef DEBUG
   debug ( "pv_release_pe -- CALLED\n");
#endif

   if ( vg == NULL || lv_pe == NULL || lv_pe->lv_num > vg->lv_max ||
        pe == NULL || *pe == 0) return -LVM_EPARA;

   l = lv_pe->lv_num - 1;
   if ( vg->lv[l]->lv_allocated_le % stripes != 0) return -LVM_ESIZE;
   p = vg->lv[l]->lv_allocated_le - 1;
   np = 0;

   /* linear mode */
   if ( stripes < 2) {
      while ( p >= 0 && np < *pe) {
         for ( pv_num = 0; pv_num < vg->pv_cur; pv_num++) {
            if ( vg->pv[pv_num]->pv_dev == vg->lv[l]->lv_current_pe[p].dev)
               break;
         }
         if ( pv_num == vg->pv_cur) return -LVM_EPV_RELEASE_PE_NO_PV;
         pe_index = ( vg->lv[l]->lv_current_pe[p].pe - 
                      LVM_DISK_SIZE ( vg->pv[pv_num]) / SECTOR_SIZE) / vg->pe_size;
#ifdef DEBUG
         debug ( "pv_release_pe -- pv_name: %s  pe: %lu  sector: %lu\n",
                  vg->pv[pv_num]->pv_name,
                  pe_index,
                  vg->lv[l]->lv_current_pe[p].pe);
#endif
         vg->pv[pv_num]->pe[pe_index].lv_num = \
         vg->pv[pv_num]->pe[pe_index].le_num = 0;
         vg->pv[pv_num]->pe_allocated--;
         vg->lv[l]->lv_current_le--;
         vg->lv[l]->lv_allocated_le--;
         np++;
         if ( lv_check_on_pv ( vg->pv[pv_num], lv_pe->lv_num) != TRUE)
            vg->pv[pv_num]->lv_cur--;
         p--;
      }

   /* striped mode */
   } else {
      while ( p >= 0 && np < *pe) {
         offset = vg->lv[l]->lv_allocated_le / stripes;
         for ( i = 0; i < *pe / stripes; i++) {
            for ( j = 1; j <= stripes; j++) {
               p = j * offset - 1 - i;
               for ( pv_num = 0; pv_num < vg->pv_cur; pv_num++) {
                  if ( vg->pv[pv_num]->pv_dev ==
                          vg->lv[l]->lv_current_pe[p].dev)
                     break;
               }
               pe_index = ( vg->lv[l]->lv_current_pe[p].pe - 
                            LVM_DISK_SIZE ( vg->pv[0]) / SECTOR_SIZE) /
                            vg->pe_size;
               vg->pv[pv_num]->pe[pe_index].lv_num = \
               vg->pv[pv_num]->pe[pe_index].le_num = 0;
               vg->pv[pv_num]->pe_allocated--;
               vg->lv[l]->lv_current_le--;
               vg->lv[l]->lv_allocated_le--;
               np++;
               if ( lv_check_on_pv ( vg->pv[pv_num], lv_pe->lv_num) != TRUE)
                  vg->pv[pv_num]->lv_cur--;
               p--;
            }
         }
         length = offset - *pe / stripes;
         for ( i = 1; i < stripes; i++) {
            source = i * offset;
            dest = source - i * ( *pe / stripes);
            for ( j = 0; j < length; j++) {
               vg->lv[l]->lv_current_pe[dest+j].dev =
                  vg->lv[l]->lv_current_pe[source+j].dev;
               vg->lv[l]->lv_current_pe[dest+j].pe =
                  vg->lv[l]->lv_current_pe[source+j].pe;
               vg->pv[pv_get_index_by_kdev_t ( vg, vg->lv[l]->lv_current_pe[dest+j].dev)]->pe[(vg->lv[l]->lv_current_pe[dest+j].pe-(vg->pv[0]->pe_on_disk.base+vg->pv[0]->pe_on_disk.size)/SECTOR_SIZE)/vg->pe_size].le_num = dest+j;
            }
         }
         if ( ( vg->lv[l]->lv_current_pe =
                realloc ( vg->lv[l]->lv_current_pe,
                          stripes * length *
                          sizeof ( pe_t))) == NULL) {
            fprintf ( stderr, "realloc error in %s [line %d]\n",
                              __FILE__, __LINE__);
            return -LVM_EPV_RELEASE_PE_REALLOC;
         }
      }
   }

   *pe -= np;

#ifdef DEBUG
   debug ( "pv_release_pe -- LEAVING\n");
#endif
   return 0;
}

==> lvm-0.4alpha.tar.gz: LVM/0.4alpha/tools/lib/pv_reserve_release_pe.c <==
/*
 * Author: Heinz Mauelshagen, Germany
 *
 * March 1997
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <liblvm.h>

int pv_reserve_pe ( pv_t *pv, disk_pe_t *lv_pe, uint *pe,
                    pe_t *pe_p, uint allocation, int new) {
   int p = 0;
   int np = 0;

#ifdef DEBUG
   debug ( "pv_reserve_pe -- CALLED: pv->pv_dev: %02d:%02d  lv_num: %d"
            "  le_num: %d  pv->pe_total: %lu\n",
             MAJOR(pv->pv_dev),
             MINOR(pv->pv_dev),
             lv_pe->lv_num,
             lv_pe->le_num,
             pv->pe_total);
#endif

   if ( pv == NULL || lv_pe == NULL || lv_pe->lv_num > MAX_LV ||
        pe == NULL || *pe == 0 || pe_p == NULL ||
        ( allocation != 0 && allocation != LV_CONTIGUOUS) ||
        ( new != TRUE && new != FALSE)) return -LVM_EPARAM;

   p = np = 0;
   if ( ( allocation & LV_CONTIGUOUS) && new != TRUE) {
      p = pv->pe_total - 1;
      while ( pv->pe[p].lv_num != lv_pe->lv_num && p > -1) p--;
      if ( p > 0) p++;
      else return -LVM_EPV_RESERVE_PE_LV; /* this LV not on this PV */
   }

   for ( p = 0; p < pv->pe_total && np < *pe; p++) {
      if ( pv->pe[p].lv_num == 0) {
#ifdef DEBUG
         debug ( "pv_reserve_pe -- empty PE %d\n", p);
#endif
         if ( ( allocation & LV_CONTIGUOUS) && new != TRUE &&
                pv->pe[p-1].lv_num != lv_pe->lv_num) return -LVM_ESIZE;
         pv->pe[p].lv_num = lv_pe->lv_num;
         pv->pe[p].le_num = lv_pe->le_num;
         lv_pe->le_num++;
         pv->pe_allocated++;
         pe_p->dev = pv->pv_dev;
         pe_p->pe = ( p * pv->pe_size) + LVM_DISK_SIZE ( pv) / SECTOR_SIZE;
         pe_p++;
         np++;
      }
   }
   *pe -= np;

#ifdef DEBUG
   debug ( "pv_reserve_pe -- LEAVING\n");
#endif
   return 0;
}


int pv_release_pe ( vg_t *vg, disk_pe_t *lv_pe, uint *pe, uint stripes) {
   int l = 0;
   int pv_num = 0;
   uint dest = 0;
   uint i = 0;
   uint j = 0;
   uint length = 0;
   uint np = 0;
   uint offset = 0;
   uint pe_index = 0;
   uint source = 0;
   uint p = 0;

#ifdef DEBUG
   debug ( "pv_release_pe -- CALLED\n");
#endif

   if ( vg == NULL || lv_pe == NULL || lv_pe->lv_num > vg->lv_max ||
        pe == NULL || *pe == 0) return -LVM_EPARAM;

   l = lv_pe->lv_num - 1;
   if ( vg->lv[l]->lv_allocated_le % stripes != 0) return -LVM_ESIZE;
   p = vg->lv[l]->lv_allocated_le - 1;
   np = 0;

   /* linear mode */
   if ( stripes < 2) {
      while ( p >= 0 && np < *pe) {
         for ( pv_num = 0; pv_num < vg->pv_cur; pv_num++) {
            if ( vg->pv[pv_num]->pv_dev == vg->lv[l]->lv_current_pe[p].dev)
               break;
         }
         if ( pv_num == vg->pv_cur) return -LVM_EPV_RELEASE_PE_NO_PV;
         pe_index = ( vg->lv[l]->lv_current_pe[p].pe - 
                      LVM_DISK_SIZE ( vg->pv[pv_num]) / SECTOR_SIZE) / vg->pe_size;
#ifdef DEBUG
         debug ( "pv_release_pe -- pv_name: %s  pe: %lu  sector: %lu\n",
                  vg->pv[pv_num]->pv_name,
                  pe_index,
                  vg->lv[l]->lv_current_pe[p].pe);
#endif
         vg->pv[pv_num]->pe[pe_index].lv_num = \
         vg->pv[pv_num]->pe[pe_index].le_num = 0;
         vg->pv[pv_num]->pe_allocated--;
         vg->lv[l]->lv_current_le--;
         vg->lv[l]->lv_allocated_le--;
         np++;
         if ( lv_check_on_pv ( vg->pv[pv_num], lv_pe->lv_num) != TRUE)
            vg->pv[pv_num]->lv_cur--;
         p--;
      }

   /* striped mode */
   } else {
      while ( p >= 0 && np < *pe) {
         offset = vg->lv[l]->lv_allocated_le / stripes;
         for ( i = 0; i < *pe / stripes; i++) {
            for ( j = 1; j <= stripes; j++) {
               p = j * offset - 1 - i;
               for ( pv_num = 0; pv_num < vg->pv_cur; pv_num++) {
                  if ( vg->pv[pv_num]->pv_dev ==
                          vg->lv[l]->lv_current_pe[p].dev)
                     break;
               }
               pe_index = ( vg->lv[l]->lv_current_pe[p].pe - 
                            LVM_DISK_SIZE ( vg->pv[0]) / SECTOR_SIZE) /
                            vg->pe_size;
               vg->pv[pv_num]->pe[pe_index].lv_num = \
               vg->pv[pv_num]->pe[pe_index].le_num = 0;
               vg->pv[pv_num]->pe_allocated--;
               vg->lv[l]->lv_current_le--;
               vg->lv[l]->lv_allocated_le--;
               np++;
               if ( lv_check_on_pv ( vg->pv[pv_num], lv_pe->lv_num) != TRUE)
                  vg->pv[pv_num]->lv_cur--;
               p--;
            }
         }
         length = offset - *pe / stripes;
         for ( i = 1; i < stripes; i++) {
            source = i * offset;
            dest = source - i * ( *pe / stripes);
            for ( j = 0; j < length; j++) {
               vg->lv[l]->lv_current_pe[dest+j].dev =
                  vg->lv[l]->lv_current_pe[source+j].dev;
               vg->lv[l]->lv_current_pe[dest+j].pe =
                  vg->lv[l]->lv_current_pe[source+j].pe;
               vg->pv[pv_get_index_by_kdev_t ( vg, vg->lv[l]->lv_current_pe[dest+j].dev)]->pe[(vg->lv[l]->lv_current_pe[dest+j].pe-(vg->pv[0]->pe_on_disk.base+vg->pv[0]->pe_on_disk.size)/SECTOR_SIZE)/vg->pe_size].le_num = dest+j;
            }
         }
         if ( ( vg->lv[l]->lv_current_pe =
                realloc ( vg->lv[l]->lv_current_pe,
                          stripes * length *
                          sizeof ( pe_t))) == NULL) {
            fprintf ( stderr, "realloc error in %s [line %d]\n",
                              __FILE__, __LINE__);
            return -LVM_EPV_RELEASE_PE_REALLOC;
         }
      }
   }

   *pe -= np;

#ifdef DEBUG
   debug ( "pv_release_pe -- LEAVING\n");
#endif
   return 0;
}
